package info.msxlaunchers.openmsx.launcher.persistence.game;

import info.msxlaunchers.openmsx.launcher.builder.GameBuilder;
import info.msxlaunchers.openmsx.launcher.data.extra.ExtraData;
import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.data.game.constants.Genre;
import info.msxlaunchers.openmsx.launcher.persistence.game.ActionDecider;
import info.msxlaunchers.openmsx.launcher.persistence.game.CSVFileGamePersister;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseMaxBackupReachedException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNullNameException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameAlreadyExistsException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameWithNullNameException;

import java.io.File;

import java.io.IOException;
import java.lang.reflect.Field;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.eq;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class CSVFileGamePersisterTest
{
	@Mock ActionDecider actionDecider;
	@Mock GameBuilder gameBuilder;

	private File databaseFile = null;
	private String database = null;
	private File databaseFile2 = null;
	private String database2 = null;
	private String currentDirectory = System.getProperty( "user.dir" );

	private File databasesDirectory = new File( currentDirectory, "databases" );
	private File backupsDirectory = new File( currentDirectory, "backups" );

	@Before
	public void prepare()
	{
		MockitoAnnotations.initMocks( this );

		database = "testDatabase";
		databaseFile = new File( databasesDirectory, database + ".dbo" );

		//delete this file if it exists
		if( databaseFile.exists() )
		{
			databaseFile.delete();
		}

		database2 = "testDatabase2";
		databaseFile2 = new File( databasesDirectory, database2 + ".dbo" );

		//delete this file if it exists
		if( databaseFile2.exists() )
		{
			databaseFile2.delete();
		}
	}

	@After
	public void cleanup()
	{
		databaseFile.delete();

		//clean up backups
		File[] files = backupsDirectory.listFiles(); 
		for ( File file: files ) 
		{
			if( file.isFile() ) 
			{
				String filename = file.getName();
				if( filename.endsWith( ".bak" ) )
				{
					file.delete();
				}
			}
		}
	}

	@Test
	public void testCreateAndDeleteDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );
		
		//check that file exists
		assertTrue( databaseFile.exists() );
		
		//delete database
		csvFilePersister.deleteDatabase( database );

		//check that file doesn't exist
		assertFalse( databaseFile.exists() );
	}

	@Test( expected = DatabaseAlreadyExistsException.class )
	public void testCreateExistingDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );
		
		//check that file exists
		assertTrue( databaseFile.exists() );
		
		//create database a second time
		csvFilePersister.createDatabase( database );
		
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testDeleteNonExistingDatabase() throws DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.deleteDatabase( "non-existing" );
	}

	@Test( expected = NullPointerException.class )
	public void testRenameDatabaseNullArg1() throws DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.renameDatabase( null, "newDatabase" );
	}

	@Test( expected = NullPointerException.class )
	public void testRenameDatabaseNullArg2() throws DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.renameDatabase( "oldDatabase", null );
	}

	@Test
	public void testCreateAndRenameDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );
		
		//check that file exists
		assertTrue( databaseFile.exists() );
		
		//rename database
		String anotherName = "anotherName";
		csvFilePersister.renameDatabase( database, anotherName );

		//check that original file doesn't exist
		assertFalse( databaseFile.exists() );

		//check that new file exists
		assertTrue( new File( databasesDirectory, anotherName + ".dbo" ).exists() );

		//rename back and delete
		csvFilePersister.renameDatabase( anotherName, database );
		csvFilePersister.deleteDatabase( database );

		//check that both files don't exist
		assertFalse( databaseFile.exists() );
		assertFalse( new File( databasesDirectory, anotherName + ".dbo" ).exists() );
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testRenameNonExistingDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.renameDatabase( "oldDatabase", "newDatabase" );
	}

	@Test
	public void testBackupDatabase()
			throws DatabaseMaxBackupReachedException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );
		
		for( int backupNumber = 1; backupNumber < 15; backupNumber++ )
		{
			csvFilePersister.backupDatabase( database );			
		}

		//check that a sample of the backup files exist
		File databaseFileBackup = new File( backupsDirectory, database + ".dbo_01.bak" );
		assertTrue( databaseFileBackup.exists() );

		databaseFileBackup = new File( backupsDirectory, database + ".dbo_02.bak" );
		assertTrue( databaseFileBackup.exists() );

		databaseFileBackup = new File( backupsDirectory, database + ".dbo_06.bak" );
		assertTrue( databaseFileBackup.exists() );

		databaseFileBackup = new File( backupsDirectory, database + ".dbo_09.bak" );
		assertTrue( databaseFileBackup.exists() );

		databaseFileBackup = new File( backupsDirectory, database + ".dbo_10.bak" );
		assertTrue( databaseFileBackup.exists() );

		databaseFileBackup = new File( backupsDirectory, database + ".dbo_14.bak" );
		assertTrue( databaseFileBackup.exists() );
		
		//test that backing up another database does not interfere with the other databases' backup numbers
		String database2 = "another_database";
		csvFilePersister.createDatabase( database2 );
		
		csvFilePersister.backupDatabase( database2 );

		File database2FileBackup = new File( backupsDirectory, database2 + ".dbo_01.bak" );
		assertTrue( database2FileBackup.exists() );

		csvFilePersister.deleteDatabase( database2 );

		database2FileBackup.delete();
	}

	@Test( expected = DatabaseMaxBackupReachedException.class )
	public void testBackupDatabaseMaxReached()
			throws DatabaseMaxBackupReachedException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//check that we cannot exceed 99 backups
		File databaseFileBackup = new File( backupsDirectory, database + ".dbo_98.bak" );
		databaseFileBackup.createNewFile();
		assertTrue( databaseFileBackup.exists() );

		csvFilePersister.backupDatabase( database );			
		databaseFileBackup = new File( backupsDirectory, database + ".dbo_99.bak" );
		assertTrue( databaseFileBackup.exists() );

		csvFilePersister.backupDatabase( database );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForBackupDatabase()
			throws DatabaseMaxBackupReachedException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.backupDatabase( null );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForRecreateDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.recreateDatabase( null );
	}

	@Test
	public void testRecreateDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		csvFilePersister.recreateDatabase( database );
		assertTrue( databaseFile.exists() );
	}

	@Test
	public void testGetDatabases() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		Set<String> databases;
		
		//get list of empty databases
		databases = csvFilePersister.getDatabases();
		assertEquals( 0, databases.size() );

		String database1 = "Games 1";
		String database2 = "Games 2";

		//create new database 1
		csvFilePersister.createDatabase( database1 );

		//create new database 2
		csvFilePersister.createDatabase( database2 );

		//get list of two databases
		databases = csvFilePersister.getDatabases();
		assertEquals( 2, databases.size() );

		//check that both databases are in the list
		assertTrue( databases.contains( database1 ) );
		assertTrue( databases.contains( database2 ) );

		//delete the databases
		csvFilePersister.deleteDatabase( database1 );

		csvFilePersister.deleteDatabase( database2 );

		//no databases are left
		databases = csvFilePersister.getDatabases();
		assertEquals( 0, databases.size() );
	}

	@Test
	public void testGetDatabasesSetCanBeModified() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//get list of empty databases
		Set<String> databases = csvFilePersister.getDatabases();

		//add a dummy database to the Set to see if we can modify it
		databases.add( "something" );
	}

	@Test( expected = NullPointerException.class )
	public void getUpdateGameExtraDataInDatabasesNullArg() throws IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.updateGameExtraDataInDatabases( null );
	}

	@Test
	public void getUpdateGameExtraDataInDatabases() throws DatabaseNullNameException, DatabaseAlreadyExistsException, IOException, DatabaseNotFoundException, GameWithNullNameException, GameAlreadyExistsException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		String database1 = "Games 1";
		String database2 = "Games 2";

		//create two new databases
		csvFilePersister.createDatabase( database1 );
		csvFilePersister.createDatabase( database2 );

		//add games to one of the databases
		Game game1 = Game.name( "game1" ).romA( "romA" ).machine( "machine" ).build();
		Game game2 = Game.name( "game2" ).diskA( "diskA" ).machine( "machine" ).build();
		Game game3 = Game.name( "game3" ).tape( "tape" ).machine( "machine" ).build();

		Game newGame1 = Game.name( "game1" ).romA( "romA" ).machine( "machine" ).isMSX2( true ).build();
		Game newGame2 = Game.name( "game2" ).diskA( "diskA" ).machine( "machine" ).isMIDI( true ).build();
		Game newGame3 = Game.name( "game3" ).tape( "tape" ).machine( "machine" ).genre1( Genre.COMMUNICATION ).build();

		csvFilePersister.saveGame( game1, database1 );
		csvFilePersister.saveGame( game2, database1 );
		csvFilePersister.saveGame( game3, database2 );

		//call the method under test
		Map<String,ExtraData> extraDataMap = Collections.emptyMap();

		when( gameBuilder.createGameObjectFromGameAndUpdateExtraData( game1, extraDataMap ) ).thenReturn( newGame1 );
		when( gameBuilder.createGameObjectFromGameAndUpdateExtraData( game2, extraDataMap ) ).thenReturn( newGame2 );
		when( gameBuilder.createGameObjectFromGameAndUpdateExtraData( game3, extraDataMap ) ).thenReturn( newGame3 );

		int numberUpdatedProfiles = csvFilePersister.updateGameExtraDataInDatabases( extraDataMap );
		assertEquals( 3, numberUpdatedProfiles );

		//get the games from the updated databases
		Set<Game> games1 = csvFilePersister.getGames( database1 );
		assertEquals( 2, games1.size() );
		for( Game game:games1 )
		{
			if( game.getName().equals( "game1" ) )
			{
				assertTrue( game.isMSX2() );
			}
			else if( game.getName().equals( "game2" ) )
			{
				assertTrue( game.isMIDI() );
			}
		}

		Set<Game> games2 = csvFilePersister.getGames( database2 );
		assertEquals( 1, games2.size() );
		assertEquals( Genre.COMMUNICATION, games2.iterator().next().getGenre1() );

		//delete the databases
		csvFilePersister.deleteDatabase( database1 );
		csvFilePersister.deleteDatabase( database2 );

		verify( gameBuilder, times( 3 ) ).createGameObjectFromGameAndUpdateExtraData( any( Game.class ), eq( extraDataMap ) );
	}

	@Test( expected = DatabaseNullNameException.class )
	public void testNullForCreateDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.createDatabase( null );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForDeleteDatabase() throws DatabaseNotFoundException, DatabaseNullNameException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.deleteDatabase( null );
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testGetGamesFromNonExistentDatabase() throws DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//get games
		csvFilePersister.getGames( "non-existing-database" );
	}

	@Test( expected = UnsupportedOperationException.class )
	public void testCannotModifyGamesListFromDatabase() throws DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		Set<Game> games = csvFilePersister.getGames( database );

		Game game = Game.name( "cannot-be-added" ).build();
		
		games.add( game );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForGetGames() throws DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.getGames( null );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForSaveGameArg1()
			throws GameWithNullNameException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.saveGame( null, database );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForSaveGameArg2()
			throws GameWithNullNameException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		Game game = Game.name( "name" ).build();
		csvFilePersister.saveGame( game, null );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForDeleteGameArg1()
			throws GameNotFoundException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		csvFilePersister.deleteGame( null, database );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForDeleteGameArg2()
			throws GameNotFoundException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		Game game = Game.name( "name" ).build();
		csvFilePersister.deleteGame( game, null );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForUpdateGameArg1()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		Game game = Game.name( "name" ).build();
		csvFilePersister.updateGame( null, game, database );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForUpdateGameArg2()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		Game game = Game.name( "name" ).build();
		csvFilePersister.updateGame( game, null, database );
	}

	@Test( expected = NullPointerException.class )
	public void testNullForUpdateGameArg3()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		Game game = Game.name( "name" ).build();
		csvFilePersister.updateGame( game, game, null );
	}

	@Test( expected = GameAlreadyExistsException.class )
	public void testSavingDuplicateGames()
			throws GameWithNullNameException, GameAlreadyExistsException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game1 = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();
		
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game1
		csvFilePersister.saveGame( game1, database );

		//save another game with the same name
		Game game1Again = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		csvFilePersister.saveGame( game1Again, database );
	}

	@Test( expected = GameNotFoundException.class )
	public void testDeleteNonExistingGame()
			throws GameNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//delete the game
		csvFilePersister.deleteGame( game, database );
	}

	@Test( expected = GameNotFoundException.class )
	public void testUpdateNonExistingGame()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseNullNameException, DatabaseAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Game newGame = Game.name( "testName2" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//delete the game
		csvFilePersister.updateGame( game, newGame, database );
	}

	@Test
	public void testSaveDeleteGames()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game1 = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Game game2 = Game.name( "testName2" )
				.romA( "testRomA2" )
				.diskA( "testDiskA2" )
				.machine( "testMachine2" )
				.build();
		
		Game game3 = Game.name( "testName3" )
				.romA( "testRomA3" )
				.diskA( "testDiskA3" )
				.machine( "testMachine3" )
				.build();
		
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game1
		csvFilePersister.saveGame( game1, database );

		//save game2
		csvFilePersister.saveGame( game2, database );

		//save game3
		csvFilePersister.saveGame( game3, database );

		//get games
		Set<Game> games = csvFilePersister.getGames( database );

		//check that total games is correct
		assertEquals( 3, games.size() );

		//check that the three games are what we saved
		assertTrue( games.contains( game1 ) );
		assertTrue( games.contains( game2 ) );
		assertTrue( games.contains( game3 ) );

		//check that string fields that weren't in the original Game object are returned as null
		assertNull( game1.getDiskB() );
		assertNull( game1.getExtensionRom() );
		assertNull( game1.getHarddisk() );
		assertNull( game1.getInfo() );
		assertNull( game1.getLaserdisc() );
		assertNull( game1.getRomB() );
		assertNull( game1.getScreenshotSuffix() );
		assertNull( game1.getSha1Code() );
		assertNull( game1.getTape() );
		assertNull( game1.getTclScript() );

		//delete game1
		csvFilePersister.deleteGame( game1, database );
		
		//get games
		games = csvFilePersister.getGames( database );

		//check that total games is correct
		assertEquals( 2, games.size() );

		//check that the remaining games are game2 and game3
		assertTrue( games.contains( game2 ) );
		assertTrue( games.contains( game3 ) );

		//delete last two games
		Set<Game> gamesToDelete = new HashSet<Game>( games );
		csvFilePersister.deleteGames( gamesToDelete, database );
		
		//get games
		games = csvFilePersister.getGames( database );

		//check that total games is correct
		assertEquals( 0, games.size() );

		//delete database
		csvFilePersister.deleteDatabase( database );
	}

	@Test
	public void testSaveGames()
			throws GameWithNullNameException, DatabaseAlreadyExistsException, GameAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game1 = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Game game2 = Game.name( "testName2" )
				.romA( "testRomA2" )
				.diskA( "testDiskA2" )
				.machine( "testMachine2" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		Set<Game> games = new HashSet<Game>();
		games.add( game1 );
		games.add( game2 );

		csvFilePersister.saveGames( games, database );

		//delete database
		csvFilePersister.deleteDatabase( database );
	}

	@Test( expected = GameAlreadyExistsException.class )
	public void testUpdatingGameUsingExistingName()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game1 = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Game game2 = Game.name( "testName2" )
				.romA( "testRomA2" )
				.diskA( "testDiskA2" )
				.machine( "testMachine2" )
				.build();

		Game game1Again = Game.name( "testName1" )
				.romA( "testRomA1'" )
				.diskA( "testDiskA1'" )
				.machine( "testMachine1'" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game1
		csvFilePersister.saveGame( game1, database );

		//save game2
		csvFilePersister.saveGame( game2, database );

		//update game2 to game1Again
		csvFilePersister.updateGame( game2, game1Again, database );
	}

	@Test
	public void testUpdateGames()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game1 = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Game game2 = Game.name( "testName2" )
				.romA( "testRomA2" )
				.diskA( "testDiskA2" )
				.machine( "testMachine2" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game1
		csvFilePersister.saveGame( game1, database );

		//save game2
		csvFilePersister.saveGame( game2, database );

		Game game3 = Game.name( "testName3" )
				.romA( "testRomA3" )
				.diskA( "testDiskA3" )
				.machine( "testMachine3" )
				.build();

		//update game1 to game3
		csvFilePersister.updateGame( game1, game3, database );

		//get games
		Set<Game> games = csvFilePersister.getGames( database );

		//check that total games is correct
		assertEquals( 2, games.size() );

		//check that the remaining games are game2 and game3
		assertTrue( games.contains( game2 ) );
		assertTrue( games.contains( game3 ) );
	}

	@Test( expected = GameWithNullNameException.class )
	public void testSaveGameWithNullName()
			throws GameWithNullNameException, GameAlreadyExistsException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game = Game.info( "testInfo1" )
				.machine( "testMachine" )
				.romA( "testRomA" )
				.build();
		
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game
		csvFilePersister.saveGame( game, database );
	}

	@Test( expected = GameWithNullNameException.class )
	public void testUpdateGameWithNullName()
			throws GameWithNullNameException, GameAlreadyExistsException, GameNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException
	{
		Game game = Game.name( "name" )
				.info( "testInfo1" )
				.machine( "testMachine" )
				.romA( "testRomA" )
				.build();
		
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game
		csvFilePersister.saveGame( game, database );

		Game newGame = Game.info( "testInfo2" )
				.machine( "testMachine2" )
				.romA( "testRom2A" )
				.build();
		
		//try to update game
		csvFilePersister.updateGame( game, newGame, database );
	}

	@Test
	public void testWorkingSaveGame()
			throws GameWithNullNameException, GameAlreadyExistsException, DatabaseAlreadyExistsException, DatabaseNullNameException, DatabaseNotFoundException, IOException, IllegalAccessException
	{
		Game game = Game.name( "testName" )
				.info( "testInfo1" )
				.machine( "testMachine" )
				.romA( "testRomA" )
				.extensionRom( "scc" )
				.romB( "testRomB" )
				.diskA( "testDiskA" )
				.diskB( "testDiskB" )
				.tape( "testTape" )
				.harddisk( "testHarddiskPrimary" )
				.laserdisc( "laserdisc" )
				.tclScript( "script" )
				.isMSX( true )
				.isMSX2( true )
				.isMSX2Plus( true )
				.isTurboR( true )
				.isPSG( true )
				.isSCC( true )
				.isSCCI( true )
				.isPCM( true )
				.isMSXMUSIC( true )
				.isMSXAUDIO( true )
				.isMoonsound( true )
				.isMIDI( true )
				.genre1( Genre.ADVENTURE_ALL )
				.genre2( Genre.BEAT_EM_UP )
				.screenshotSuffix( "-en" )
				.msxGenID( 20 )
				.sha1Code( "testSha1Code" )
				.msxGenID( 4567 )
				.size( 1024 )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );
		
		//create new database
		csvFilePersister.createDatabase( database );

		//save game
		csvFilePersister.saveGame( game, database );
		
		//get list of games
		Set<Game> games = csvFilePersister.getGames( database );
		
		Game savedGame = games.iterator().next();

		//test that game is the same as savedGame
		Field fields[] = game.getClass().getDeclaredFields();
		for ( Field field : fields )
		{
			field.setAccessible( true );
			assertEquals( field.get( game ), field.get( savedGame ) );
		}
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testSaveGameInNonExistingDatabase()
			throws GameWithNullNameException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		String nonExistentDatabase = "nonExistenDatabase";
		
		//save game1 in a non-existent database
		csvFilePersister.saveGame( game, nonExistentDatabase );
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testDeleteGameFromNonExistingDatabase()
			throws GameNotFoundException, DatabaseNotFoundException, IOException
	{
		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		String nonExistentDatabase = "nonExistenDatabase";
		
		//delete game in a non-existent database
		csvFilePersister.deleteGame( game, nonExistentDatabase );
	}

	@Test( expected = DatabaseNotFoundException.class )
	public void testSaveGameInWrongDatabase()
			throws GameWithNullNameException, GameNotFoundException, GameAlreadyExistsException, DatabaseNotFoundException, IOException
	{
		Game game1 = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Game game2 = Game.name( "testName2" )
				.romA( "testRomA2" )
				.diskA( "testDiskA2" )
				.machine( "testMachine2" )
				.build();

		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		String nonExistentDatabase = "nonExistenDatabase";

		//update game1 to game2 in a non-existent database
		csvFilePersister.updateGame( game1, game2, nonExistentDatabase );
	}

	@Test( expected = NullPointerException.class )
	public void testMoveGamesNullArg1()
		throws GameWithNullNameException, DatabaseNotFoundException, GameNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.moveGames( null, database, database2, actionDecider );
	}

	@Test( expected = NullPointerException.class )
	public void testMoveGamesNullArg2()
		throws GameWithNullNameException, DatabaseNotFoundException, GameNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.moveGames( new HashSet<Game>(), null, database2, actionDecider );
	}

	@Test( expected = NullPointerException.class )
	public void testMoveGamesNullArg3()
		throws GameWithNullNameException, DatabaseNotFoundException, GameNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.moveGames( new HashSet<Game>(), database, null, actionDecider );
	}

	@Test( expected = NullPointerException.class )
	public void testMoveGamesNullArg4()
		throws GameWithNullNameException, DatabaseNotFoundException, GameNotFoundException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		csvFilePersister.moveGames( new HashSet<Game>(), database, database2, null );
	}

	@Test
	public void testMoveGamesPrmptingUserForActionYesAllFalse()
			throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Set<Game> games = new HashSet<Game>();
		games.add( game );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game, database2 );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game, database );

		when( actionDecider.isYesAll() ).thenReturn( false );
		when( actionDecider.isYes() ).thenReturn( true );
		csvFilePersister.moveGames( games, database, database2, actionDecider );

		//check that promptForAction was called once
		verify( actionDecider, times( 1 ) ).promptForAction( "testName1" );
	}

	@Test
	public void testMoveGamesPrmptingUserForActionYesAllTrue()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Set<Game> games = new HashSet<Game>();
		games.add( game );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game, database2 );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game, database );

		when( actionDecider.isYesAll() ).thenReturn( true );
		csvFilePersister.moveGames( games, database, database2, actionDecider );

		//check that promptForAction was not called
		verify( actionDecider, never() ).promptForAction( null );
	}

	@Test
	public void testMoveGamesPrmptingUserForActionNoAllFalse()
			throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Set<Game> games = new HashSet<Game>();
		games.add( game );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game, database2 );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game, database );

		when( actionDecider.isNoAll() ).thenReturn( false );
		when( actionDecider.isNo() ).thenReturn( true );
		csvFilePersister.moveGames( games, database, database2, actionDecider );

		//check that promptForAction was called once
		verify( actionDecider, times( 1 ) ).promptForAction( "testName1" );
	}

	@Test
	public void testMoveGamesPrmptingUserForActionNoAllTrue()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game = Game.name( "testName1" )
				.romA( "testRomA1" )
				.diskA( "testDiskA1" )
				.machine( "testMachine1" )
				.build();

		Set<Game> games = new HashSet<Game>();
		games.add( game );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game, database2 );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game, database );

		when( actionDecider.isNoAll() ).thenReturn( true );
		csvFilePersister.moveGames( games, database, database2, actionDecider );

		//check that promptForAction was not called
		verify( actionDecider, never() ).promptForAction( null );
	}

	@Test
	public void testMoveGamesPrmptingUserForActionYesTrue()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game1a = Game.name( "testName1" )
				.romA( "testRomA1a" )
				.diskA( "testDiskA1a" )
				.machine( "testMachine1a" )
				.build();

		Game game1b = Game.name( "testName1" )
				.romA( "testRomA1b" )
				.diskA( "testDiskA1b" )
				.machine( "testMachine1b" )
				.build();

		Game game2a = Game.name( "testName2a" )
				.romA( "testRomA2a" )
				.diskA( "testDiskA2a" )
				.machine( "testMachine2a" )
				.build();

		Game game2b = Game.name( "testName2b" )
				.romA( "testRomA2b" )
				.diskA( "testDiskA2b" )
				.machine( "testMachine2b" )
				.build();

		Set<Game> gamesToMove = new HashSet<Game>();
		gamesToMove.add( game1a );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game1a, database );
		csvFilePersister.saveGame( game2a, database );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game1b, database2 );
		csvFilePersister.saveGame( game2b, database2 );

		//need to get the games from the old "current" database to initialise the internal cache
		Set<Game> games = csvFilePersister.getGames( database );

		when( actionDecider.isYes() ).thenReturn( true );
		csvFilePersister.moveGames( gamesToMove, database, database2, actionDecider );

		//now get the games from both databases
		games = csvFilePersister.getGames( database );

		//this database contains only one game
		assertEquals( 1, games.size() );
		assertEquals( "testName2a", games.iterator().next().getName() );

		games = csvFilePersister.getGames( database2 );

		//this database contains two games
		assertEquals( 2, games.size() );
		assertTrue( games.contains( game1a ) );
		assertTrue( games.contains( game2b ) );

		//make sure that game1a was really moved
		for( Game game: games)
		{
			if( game.getName().equals( "testName1" ) )
			{
				assertEquals( "testRomA1a", game.getRomA() );
				assertEquals( "testDiskA1a", game.getDiskA() );
				assertEquals( "testMachine1a", game.getMachine() );
				break;
			}
		}
	}

	@Test
	public void testMoveGamesPrmptingUserForActionNoTrue()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game1a = Game.name( "testName1" )
				.romA( "testRomA1a" )
				.diskA( "testDiskA1a" )
				.machine( "testMachine1a" )
				.build();

		Game game1b = Game.name( "testName1" )
				.romA( "testRomA1b" )
				.diskA( "testDiskA1b" )
				.machine( "testMachine1b" )
				.build();

		Game game2a = Game.name( "testName2a" )
				.romA( "testRomA2a" )
				.diskA( "testDiskA2a" )
				.machine( "testMachine2a" )
				.build();

		Game game2b = Game.name( "testName2b" )
				.romA( "testRomA2b" )
				.diskA( "testDiskA2b" )
				.machine( "testMachine2b" )
				.build();

		Set<Game> gamesToMove = new HashSet<Game>();
		gamesToMove.add( game1a );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game1a, database );
		csvFilePersister.saveGame( game2a, database );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game1b, database2 );
		csvFilePersister.saveGame( game2b, database2 );

		//need to get the games from the old "current" database to initialise the internal cache
		Set<Game> games = csvFilePersister.getGames( database );

		when( actionDecider.isNo() ).thenReturn( true );
		csvFilePersister.moveGames( gamesToMove, database, database2, actionDecider );

		//now get the games from both databases
		games = csvFilePersister.getGames( database );

		//this database contains two games
		assertEquals( 2, games.size() );
		assertTrue( games.contains( game1a ) );
		assertTrue( games.contains( game2a ) );

		games = csvFilePersister.getGames( database2 );

		//this database contains two games
		assertEquals( 2, games.size() );
		assertTrue( games.contains( game1b ) );
		assertTrue( games.contains( game2b ) );
	}

	@Test
	public void testMoveGamesPrmptingUserForActionCancelTrue()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game1a = Game.name( "testName1" )
				.romA( "testRomA1a" )
				.diskA( "testDiskA1a" )
				.machine( "testMachine1a" )
				.build();

		Game game1b = Game.name( "testName1" )
				.romA( "testRomA1b" )
				.diskA( "testDiskA1b" )
				.machine( "testMachine1b" )
				.build();

		Game game2a = Game.name( "testName2a" )
				.romA( "testRomA2a" )
				.diskA( "testDiskA2a" )
				.machine( "testMachine2a" )
				.build();

		Game game2b = Game.name( "testName2b" )
				.romA( "testRomA2b" )
				.diskA( "testDiskA2b" )
				.machine( "testMachine2b" )
				.build();

		Set<Game> gamesToMove = new HashSet<Game>();
		gamesToMove.add( game1a );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game1a, database );
		csvFilePersister.saveGame( game2a, database );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game1b, database2 );
		csvFilePersister.saveGame( game2b, database2 );

		//need to get the games from the old "current" database to initialise the internal cache
		Set<Game> games = csvFilePersister.getGames( database );

		when( actionDecider.isCancel() ).thenReturn( true );
		csvFilePersister.moveGames( gamesToMove, database, database2, actionDecider );

		//now get the games from both databases
		games = csvFilePersister.getGames( database );

		//this database contains two games
		assertEquals( 2, games.size() );
		assertTrue( games.contains( game1a ) );
		assertTrue( games.contains( game2a ) );

		games = csvFilePersister.getGames( database2 );

		//this database contains two games
		assertEquals( 2, games.size() );
		assertTrue( games.contains( game1b ) );
		assertTrue( games.contains( game2b ) );
	}

	@Test( expected = RuntimeException.class )
	public void testMoveGamesPrmptingUserNoAction()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game1a = Game.name( "testName1" )
				.romA( "testRomA1a" )
				.diskA( "testDiskA1a" )
				.machine( "testMachine1a" )
				.build();

		Game game1b = Game.name( "testName1" )
				.romA( "testRomA1b" )
				.diskA( "testDiskA1b" )
				.machine( "testMachine1b" )
				.build();

		Set<Game> gamesToMove = new HashSet<Game>();
		gamesToMove.add( game1a );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game1a, database );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game1b, database2 );

		//need to get the games from the old "current" database to initialise the internal cache
		csvFilePersister.getGames( database );

		csvFilePersister.moveGames( gamesToMove, database, database2, actionDecider );
	}

	@Test
	public void testMoveGamesNoOverlap()
		throws GameWithNullNameException, DatabaseNotFoundException, DatabaseAlreadyExistsException, DatabaseNullNameException, GameNotFoundException, GameAlreadyExistsException, IOException
	{
		CSVFileGamePersister csvFilePersister = new CSVFileGamePersister( currentDirectory, gameBuilder );

		Game game1a = Game.name( "testName1a" )
				.romA( "testRomA1a" )
				.diskA( "testDiskA1a" )
				.machine( "testMachine1a" )
				.build();

		Game game1b = Game.name( "testName1b" )
				.romA( "testRomA1b" )
				.diskA( "testDiskA1b" )
				.machine( "testMachine1b" )
				.build();

		Game game2a = Game.name( "testName2a" )
				.romA( "testRomA2a" )
				.diskA( "testDiskA2a" )
				.machine( "testMachine2a" )
				.build();

		Game game2b = Game.name( "testName2b" )
				.romA( "testRomA2b" )
				.diskA( "testDiskA2b" )
				.machine( "testMachine2b" )
				.build();

		Set<Game> gamesToMove = new HashSet<Game>();
		gamesToMove.add( game1a );

		csvFilePersister.createDatabase( database );
		csvFilePersister.saveGame( game1a, database );
		csvFilePersister.saveGame( game2a, database );

		csvFilePersister.createDatabase( database2 );
		csvFilePersister.saveGame( game1b, database2 );
		csvFilePersister.saveGame( game2b, database2 );

		//need to get the games from the old "current" database to initialise the internal cache
		Set<Game> games = csvFilePersister.getGames( database );

		Set<Game> movedGames = csvFilePersister.moveGames( gamesToMove, database, database2, actionDecider );

		assertEquals( 1, movedGames.size() );
		assertTrue( movedGames.contains( game1a ) );

		//now get the games from both databases
		games = csvFilePersister.getGames( database );

		//this database contains one game
		assertEquals( 1, games.size() );
		assertTrue( games.contains( game2a ) );

		games = csvFilePersister.getGames( database2 );

		//this database contains three games
		assertEquals( 3, games.size() );
		assertTrue( games.contains( game1a ) );
		assertTrue( games.contains( game1b ) );
		assertTrue( games.contains( game2b ) );
	}
}