package info.msxlaunchers.openmsx.launcher.ui.presenter;

import info.msxlaunchers.openmsx.game.repository.RepositoryData;
import info.msxlaunchers.openmsx.launcher.data.game.Game;
import info.msxlaunchers.openmsx.launcher.data.settings.Settings;
import info.msxlaunchers.openmsx.launcher.extra.ExtraDataGetter;
import info.msxlaunchers.openmsx.launcher.persistence.filter.FilterPersister;
import info.msxlaunchers.openmsx.launcher.persistence.game.DatabaseNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GameNotFoundException;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersister;
import info.msxlaunchers.openmsx.launcher.persistence.settings.SettingsPersister;
import info.msxlaunchers.openmsx.launcher.starter.EmulatorStarter;
import info.msxlaunchers.openmsx.launcher.ui.presenter.GameLabel;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherException;
import info.msxlaunchers.openmsx.launcher.ui.presenter.MainPresenterImpl;
import info.msxlaunchers.openmsx.launcher.ui.view.MainView;
import info.msxlaunchers.platform.FileLocator;

import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;


import org.junit.Test;
import org.junit.runner.RunWith;

import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import com.google.inject.Provider;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anySetOf;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.when;
import static org.mockito.Mockito.verify;

@RunWith( MockitoJUnitRunner.class )
public class MainPresenterImplTest
{
	@Mock MainView view;
	@Mock Provider<SettingsPresenter> settingsPresenterFactory;
	@Mock ProfileEditingPresenterFactory profileEditingFactory;
	@Mock Provider<ScannerPresenter> scannerPresenterFactor;
	@Mock Provider<FilterEditingPresenter> filterEditingPresenterFactory;
	@Mock Provider<GamePropertiesPresenter> gamePropertiesPresenterFactory;
	@Mock Provider<BlueMSXLauncherDatabasesImporterPresenter> blueMSXLauncherImporterPresenterFactory;
	@Mock Provider<DatabaseManagerPresenter> databaseManagerPresenterFactory;
	@Mock SettingsPersister settingsPersister;
	@Mock GamePersister gamePersister;
	@Mock EmulatorStarter emulatorStarter;
	@Mock ExtraDataGetter extraDataGetter;
	@Mock RepositoryData repositoryData;
	@Mock FileLocator fileLocator;
	@Mock FilterPersister filterPersister;

	private final String defaultDatabase = "defaultDatabase";

	MainPresenterImpl presenter;

	@Test( expected = IOException.class )
	public void testWhenSettingsPersisterThrowsException() throws IOException
	{
		when( settingsPersister.getSettings() ).thenThrow( new IOException() );

		new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );
	}

	@Test
	public void testDefaultDatabaseNullInConstructor() throws IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, null, null ) );

		//no exception is thrown
		new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );
	}

	@Test
	public void testDatabaseNotFoundWhenRetrieveGamesInConstructor() throws DatabaseNotFoundException, IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( defaultDatabase ) ).thenThrow( new DatabaseNotFoundException( defaultDatabase ) );

		//no exception is thrown
		new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );
	}

	@Test
	public void testIOExceptionWhenRetrieveGamesInConstructor() throws DatabaseNotFoundException, IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( defaultDatabase ) ).thenThrow( new IOException() );

		//no exception is thrown
		new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );
	}

	@Test
	public void testStart() throws IOException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.start();
	}

	@Test
	public void testOnRequestSettingsScreen() throws IOException
	{
		SettingsPresenter settingsPresenter = mock( SettingsPresenter.class );
		when( settingsPresenterFactory.get() ).thenReturn( settingsPresenter );
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.onRequestSettingsScreen();
	}

	@Test( expected = LauncherException.class )
	public void testOnSelectDatabaseIOExceptionWhenRetrieveGames()
			throws DatabaseNotFoundException, IOException, LauncherException
	{
		String newDatabase = "newDatabase";

		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( newDatabase ) ).thenThrow( new IOException() );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.onSelectDatabase( newDatabase );
	}

	@Test( expected = LauncherException.class )
	public void testOnSelectDatabaseDatabaseNotFoundExceptionWhenRetrieveGames()
			throws DatabaseNotFoundException, IOException, LauncherException
	{
		String newDatabase = "newDatabase";

		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getGames( newDatabase ) ).thenThrow( new DatabaseNotFoundException( newDatabase ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.onSelectDatabase( newDatabase );
	}

	@Test
	public void testOnSelectDatabase() throws IOException, LauncherException
	{
		String newDatabase = "newDatabase";

		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.onSelectDatabase( newDatabase );

		verify( view, times( 1 ) ).fillGameList( newDatabase, new LinkedHashSet<GameLabel>(), null );
	}

	@Test
	public void testOnSelectDatabaseDatabaseNullOrSameDatabase() throws IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getDatabases() ).thenReturn( Collections.singleton( defaultDatabase ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		//case 1: null database
		presenter.onSelectDatabase( null );

		verify( view, never() ).fillGameList( null, new LinkedHashSet<GameLabel>(), null );
		verify( view, never() ).updateGameCount( 0 );

		//case 2: same database
		presenter.onSelectDatabase( defaultDatabase );

		verify( view, never() ).fillGameList( defaultDatabase, new LinkedHashSet<GameLabel>(), null );
		verify( view, never() ).updateGameCount( 0 );
	}

	@Test
	public void testOnLaunchGame()
			throws IOException, LauncherException
	{
		Settings settings = new Settings( null, null, null, defaultDatabase, null );

		when( settingsPersister.getSettings() ).thenReturn( settings );
		when( gamePersister.getDatabases() ).thenReturn( Collections.singleton( defaultDatabase ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.onLaunchGame( "gameName" );
	}

	@Test( expected = LauncherException.class )
	public void testOnLaunchGameExceptionWhenStartEmulator()
			throws IOException, LauncherException
	{
		Settings settings = new Settings( null, null, null, defaultDatabase, null );

		when( emulatorStarter.start( (Settings)any(), (Game)any() ) ).thenThrow( new IOException() );
		when( settingsPersister.getSettings() ).thenReturn( settings );
		when( gamePersister.getDatabases() ).thenReturn( Collections.singleton( defaultDatabase ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		presenter.onLaunchGame( "gameName" );
	}

	@Test( expected = LauncherException.class )
	public void testOnRemoveGamesIOException()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		doThrow( new IOException() ).when( gamePersister ).deleteGames( anySetOf( Game.class ), anyString() );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		Set<String> gameNames = new HashSet<String>();
		presenter.onRequestRemoveGamesAction( gameNames );
	}

	@Test( expected = LauncherException.class )
	public void testOnRemoveGamesDatabaseDatabaseNotFoundException()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		doThrow( new DatabaseNotFoundException( defaultDatabase ) ).when( gamePersister ).deleteGames( anySetOf( Game.class ), anyString() );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		Set<String> gameNames = new HashSet<String>();
		presenter.onRequestRemoveGamesAction( gameNames );
	}

	@Test
	public void testOnRemoveGameFromNonExistentDatabase()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getDatabases() ).thenReturn( Collections.singleton( defaultDatabase ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		Set<String> gameNames = new HashSet<String>();
		gameNames.add( "gameName" );

		doThrow( new GameNotFoundException( "gameName" ) ).when( gamePersister ).deleteGames( anySetOf( Game.class ), anyString() );

		presenter.onRequestRemoveGamesAction( gameNames );

		verify( view, times( 1 ) ).updateGameCount( 0 );
	}

	@Test
	public void testOnRemoveGamesDatabase()
			throws GameNotFoundException, DatabaseNotFoundException, IOException, LauncherException
	{
		when( settingsPersister.getSettings() ).thenReturn( new Settings( null, null, null, defaultDatabase, null ) );
		when( gamePersister.getDatabases() ).thenReturn( Collections.singleton( defaultDatabase ) );

		MainPresenterImpl presenter = new MainPresenterImpl( view, settingsPresenterFactory, profileEditingFactory, scannerPresenterFactor, filterEditingPresenterFactory, gamePropertiesPresenterFactory, blueMSXLauncherImporterPresenterFactory, databaseManagerPresenterFactory, settingsPersister, gamePersister, emulatorStarter, extraDataGetter, repositoryData, fileLocator, filterPersister );

		Set<String> gameNames = new HashSet<String>();
		gameNames.add( "gameName" );

		presenter.onRequestRemoveGamesAction( gameNames );

		verify( view, times( 1 ) ).updateGameCount( 0 );
	}
}