/*
 * Copyright 2014 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.view.swing;

import info.msxlaunchers.openmsx.launcher.data.settings.constants.Language;
import info.msxlaunchers.openmsx.launcher.ui.presenter.DatabaseManagerPresenter;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherException;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.component.MessageBoxUtil;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.component.JTableButtonColumn;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.images.Icons;
import info.msxlaunchers.openmsx.launcher.ui.view.swing.language.LanguageDisplayFactory;

import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

/**
 * Database Manager window class
 * 
 * @since v1.3
 * @author Sam Elsharif
 *
 */
@SuppressWarnings("serial")
public class DatabaseManagerWindow extends JDialog implements ActionListener
{
	private final DatabaseManagerPresenter presenter;
	private final Map<String,String> messages;
	private final boolean rightToLeft;
	private final Component parent;
	private final Set<String> databases;

	private final DatabaseManagerWindow thisWindow = this;

	private JTable table;
	private TableModel tableModel;
	private JButton okButton;

	public DatabaseManagerWindow(DatabaseManagerPresenter presenter,
			Language language,
			boolean rightToLeft,
			Set<String> databases)
	{
		this.presenter = presenter;
		this.rightToLeft = rightToLeft;

		this.parent = GlobalSwingContext.getIntance().getMainWindow();
		this.messages = LanguageDisplayFactory.getDisplayMessages(getClass(), language);
		this.databases = databases;
	}

	public void display()
	{
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setTitle(messages.get("DATABASE_MANAGER"));
		setModalityType(ModalityType.APPLICATION_MODAL);
		setResizable(false);

		JPanel contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);

		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));

		JPanel tablePane = new JPanel();

		tableModel = new TableModel();
		table = new JTable(tableModel);
		table.setRowSelectionAllowed(false);
		table.getTableHeader().setResizingAllowed(false);
		table.getTableHeader().setReorderingAllowed(false);
		table.setPreferredScrollableViewportSize(new Dimension(350, 82));
		table.getColumnModel().getColumn(0).setPreferredWidth(330);
		table.getColumnModel().getColumn(1).setPreferredWidth(10);
		table.getColumnModel().getColumn(2).setPreferredWidth(10);

		Action editAction = new AbstractAction()			
		{
			@Override
		    public void actionPerformed(ActionEvent e)
		    {
				try
				{
					editTableRow(Integer.parseInt(e.getActionCommand()));
				}
				catch(LauncherException le)
				{
					MessageBoxUtil.showErrorMessageBox(thisWindow, le, messages, rightToLeft);
				}
		    }
		};
		new JTableButtonColumn(table, editAction, 1, Icons.EDIT.getImageIcon());

		Action deleteAction = new AbstractAction()			
		{
			@Override
		    public void actionPerformed(ActionEvent e)
		    {
				try
				{
					removeRowFromTable(Integer.parseInt(e.getActionCommand()));
				}
				catch(LauncherException le)
				{
					MessageBoxUtil.showErrorMessageBox(thisWindow, le, messages, rightToLeft);
				}
		    }
		};
		new JTableButtonColumn(table, deleteAction, 2, Icons.DELETE.getImageIcon());

		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		tablePane.add(scrollPane);

		for(String database:databases)
		{
			tableModel.addRow(new Object[] {database});
		}

		JPanel buttonsPane = new JPanel();
		buttonsPane.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

		okButton = new JButton(messages.get("OK"));
		okButton.addActionListener(this);
		okButton.setPreferredSize(MainWindow.BUTTON_DIMENSION);
		buttonsPane.add(okButton);

		if(rightToLeft)
		{
			table.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
			((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer()).setHorizontalAlignment(JLabel.RIGHT);
			DefaultTableCellRenderer tableRenderer = new DefaultTableCellRenderer();
			tableRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
			table.getColumnModel().getColumn(0).setCellRenderer(tableRenderer);

			buttonsPane.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		}

		contentPane.add(tablePane);
		contentPane.add(buttonsPane);

		pack();
		setLocationRelativeTo(parent);
		setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource() == okButton)
		{
			dispose();
		}
	}

	private class TableModel extends DefaultTableModel
	{
		private final String[] columnNames = { messages.get("DATABASE"), "", "" };

		@Override
		public boolean isCellEditable(int row, int col)
		{
			//the last two columns - the edit and delete buttons - are editable
			return (col > 0);
		}

		@Override
		public int getColumnCount() { return columnNames.length; }

		@Override
	    public String getColumnName(int col) { return columnNames[col]; }
	}

	private void editTableRow(int index) throws LauncherException
	{
		String newName = presenter.onRequestRenameDatabaseScreen((String)tableModel.getValueAt(index, 0));

		if(newName != null)
		{
			tableModel.removeRow(index);
	
			//insert the new row in the correct alphabetical order
			int insertIndex = 0;
			while(insertIndex < tableModel.getRowCount() && ((String)tableModel.getValueAt(insertIndex, 0)).compareToIgnoreCase(newName) < 0)
			{
				insertIndex++;
			}
			tableModel.insertRow(insertIndex, new Object[] {newName});
		}
	}

	private void removeRowFromTable(int index) throws LauncherException
	{
		if(MessageBoxUtil.showYesNoMessageBox(this, messages.get("DELETE_CONFIRMATION"), messages, rightToLeft) == 0)
		{
			presenter.onRequestDeleteDatabase((String)tableModel.getValueAt(index, 0));
			tableModel.removeRow(index);
		}
	}
}